<?php

/**
 * @file
 * Functions for batch importation of Ooyala videos from Backlot.
 */

/**
 * Menu callback to list all videos from Backlot for importing as nodes.
 */
function ooyala_import_multiple(&$form_state) {
  module_load_include('inc', 'ooyala', 'includes/ooyala.api');

  // If doing a confirmation form, short cut the rest of the process.
  if (isset($form_state['storage']['import'])) {
    $form = ooyala_import_confirm($form_state);
    $form['video_info'] = array();
    $list = array();
    foreach ($form_state['storage']['import'] as $video) {
      $list[] = check_plain($video['name']);
    }
    $form['list'] = array(
      '#type' => 'markup',
      '#value' => theme('item_list', $list),
      '#weight' => -1,
    );
    $form = confirm_form($form, t('Are you sure you want to import these videos?'), NULL, '', t('Import'));
    return $form;
  }

  $form = array();
  $type = variable_get('ooyala_primary_content_type', FALSE);
  $types = ooyala_type_names();
  $videos = ooyala_api_video_query();
  if (empty($types)) {
    // If the Type is empty we won't even try to load videos.
    drupal_set_message(t('There are currently no content types that contain an Ooyala field. Add an Ooyala field under <em>Manage fields</em> from the <a href="!url">Content type configuration</a> in order to import content', array('!url' => url('admin/content/types'))), 'error');
    return $form;
  }
  if (empty($type)) {
    // If the Type is empty we won't even try to load videos.
    drupal_set_message(t('There are currently no default content types set up for video import. Set up a default content type under <em>Default content type</em> from the <a href="!url">Ooyala settings configuration</a> in order to import content', array('!url' => url('admin/settings/ooyala'))), 'error');
    return $form;
  }
  if (!empty($videos)) {
    // Fetch the statuses for all videos.
    $statuses = ooyala_get_status($type, array_keys($videos));

    foreach ($videos as $embed_code => $data) {
      // Add the video to the video info table.
      $form['video_info'][$embed_code]['title'] = array('#value' => check_plain($data['name']));
      $form['video_info'][$embed_code]['description'] = array('#value' => check_markup($data['description'], FILTER_FORMAT_DEFAULT));
      foreach (array('status') as $meta) {
        $form['video_info'][$embed_code]['metadata'][$meta] = array('#value' => check_plain($data[$meta]));
      }

      // Check if this embed_code has already been imported.
      $status = !empty($statuses[$embed_code]);
      $form['video_info'][$embed_code]['status'] = array('#value' => $status ? $status : t('Unimported'));

      // Only show the "Import" link if not yet imported.
      if ($status == FALSE) {
        $form['video_info'][$embed_code]['operations'] = array('#value' => l('Import', 'admin/content/ooyala/import/'. $embed_code));
      }
      else {
        $form['video_info'][$embed_code]['operations'] = array();
      }

      // Add the video to the list of checkboxes.
      $options[$embed_code] = check_plain($data['name']);
    }

    $form['import'] = array(
      '#type' => 'checkboxes',
      '#options' => $options,
      '#default_value' => $statuses,
      '#access' => count(ooyala_type_names()),
    );
    $form['videos'] = array(
      '#type' => 'value',
      '#value' => $videos,
    );
    $form['buttons']['submit'] = array(
      '#type' => 'submit', 
      '#value' => t('Import selected'),
      '#access' => $type,
    );
  }
  return $form;
}

/**
 * Submit handler ooyala_import().
 */
function ooyala_import_multiple_submit($form, &$form_state) {
  // If it hasn't been confirmed yet, reload the form with the confirmation.
  if (!isset($form_state['storage']['import'])) {
    $form_state['storage']['import'] = array_intersect_key($form_state['values']['videos'], array_filter($form_state['values']['import']));
    $form_state['rebuild'] = TRUE;
    return;
  }

  // After the confirmation, start the Batch API process to import the videos.
  $videos = $form_state['storage']['import'];
  $type = $form_state['values']['content_type'];
  unset($form_state['storage']['import']);

  // Batch it up.
  $batch = array(
    'operations' => array(
      array('ooyala_import_batch_process', array($videos, $type)),
    ),
    'finished' => 'ooyala_import_batch_finished',
    'title' => t('Importing videos from Ooyala Backlot'),
    'init_message' => t('Import starting.'),
    'progress_message' => t('Processed @current out of @total.'),
    'error_message' => t('Import has encountered an error.'),
    'file' => drupal_get_path('module', 'ooyala') . '/includes/ooyala.import.inc',
  );
  batch_set($batch);
}

/**
 * Theme the ooyala_import_multiple() form.
 */
function theme_ooyala_import_multiple($form) {
  // Build up a list of all the videos in a table.
  $rows = array();
  foreach (element_children($form['video_info']) as $key) {
    $data = array();

    // Fetch info.
    $title = drupal_render($form['video_info'][$key]['title']);

    // Localize theme description.
    $description = drupal_render($form['video_info'][$key]['description']);

    // Status.
    $status = drupal_render($form['video_info'][$key]['status']);

    // Meta data.
    foreach (element_children($form['video_info'][$key]['metadata']) as $meta) {
      $data[] = '<strong>'. $meta .'</strong>: '. drupal_render($form['video_info'][$key]['metadata'][$meta]);
    }
    $description .= '<div class="metadata">'. implode(', ', $data) .'</div>';

    // Combine checkbox and single import link.
    $form['import'][$key]['#title'] = NULL;
    $import = drupal_render($form['import'][$key]);
    $ops = drupal_render($form['video_info'][$key]['operations']);

    if (isset($title)) {
      // Style video info.
      $video = '';
      $video .= '<div class="ooyala-video-info">';
      $video .= '<strong><label for="'. $form['import'][$key]['#id'] .'">'. $title .'</label></strong>';
      $video .= '<div class="description">'. $description .'</div>';
      $video .= '</div>';

      if (is_numeric($status)) {
        $status = l('Imported', 'node/'. $status);
      }

      // Build rows.
      $row = array();
      $row[] = array('data' => $import, 'class' => 'checkbox');
      $row[] = $video;
      $row[] = $status;
      $row[] = array('data' => $ops);
      $rows[] = $row;
    }
  }

  $output = '';
  if ($rows) {
    $select_header = theme('table_select_header_cell');
    $select_header['class'] .= ' checkbox';
    $header = array($select_header, t('Video information'), t('Status'), t('Operations'));
  
    drupal_add_css(drupal_get_path('module', 'ooyala') . '/ooyala.css');
    $output .= theme('table', $header, $rows, array('id' => 'ooyala-import'));
  }
  elseif (isset($form['video_info'])) {
    $output .= '<p>' . t('There are no videos in your Ooyala Backlot account, or Ooyala module was unable to reach the Ooyala server.') . '</p>';
  }

  $output .= drupal_render($form);
  return $output;
}

/**
 * Menu callback for importing a single item from Backlot.
 */
function ooyala_import_confirm($form_state, $item = NULL) {
  // Allow the user to determine what type of node this should be.
  $options = ooyala_type_names();
  $default = variable_get('ooyala_primary_content_type', NULL);
  $form['content_type'] = array(
    '#type' => 'select', 
    '#title' => t('Import as'),
    '#required' => TRUE,
    '#options' => $options,
    '#default_value' => isset($options[$default]) ? $default : NULL,
    '#description' => t('This type of content will be used for the import.'),
  );

  if (empty($options)) {
    $form['content_type']['#disabled'] = TRUE;
    $form['content_type']['#options'][''] = t('No available types');
    drupal_set_message(t('There are no content types that have an Ooyala field configured. At least one Ooyala field must be added to your site before content can be imported.'), 'warning');
  }

  // If doing a batch-import, the selection form is all we need.
  if (!isset($item)) {
    return $form;
  }

  // Video information.
  $form['info'] = array(
    '#type' => 'fieldset',
    '#title' => t('Video info'),
    '#collapsable' => FALSE,
    '#weight' => 20,
  );
  foreach ($item as $key => $val) {
    $item_desc[] = '<strong>' . check_plain($key) . '</strong>: '. (is_array($val) ? check_plain(implode(', ', $val)) : check_plain($val));
  }
  $form['info']['item_desc'] = array(
    '#type' => 'markup', 
    '#value' => theme('item_list', $item_desc),
  );
  $form['item'] = array(
    '#type' => 'value', 
    '#value' => $item,
  );

  return confirm_form($form, t('Are you sure you want to import this video?'), 'admin/content/ooyala/import', '', t('Import'));
}

/**
 * Submit function for ooyala_import_confirm().
 */
function ooyala_import_confirm_submit($form, &$form_state) {
  $item = $form_state['values']['item'];
  $type = $form_state['values']['content_type'];

  // Create a node out of it.
  $node = ooyala_create_node($item, $type);

  // Pull in the thumbnail for the new node.
  ooyala_api_fetch_image($item['embed_code']);

  // Redirect to the node.
  $form_state['redirect'] = array('node/' . $node->nid);
}

/**
 * Batch processor callback.
 */
function ooyala_import_batch_process($videos, $type, &$context) {
  if (!isset($context['sandbox']['progress'])) {
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['max'] = count($videos);
    $context['sandbox']['videos'] = $videos;
  }

  // Process videos by groups of 20.
  $count = min(20, count($context['sandbox']['videos']));
  for ($i = 1; $i <= $count; $i++) {
    // For each nid, load the node, reset the values, and save it.
    $video = array_shift($context['sandbox']['videos']);
    $node = ooyala_create_node($video, $type);

    // Pull in the thumbnail for the new node.
    ooyala_api_fetch_image($video['embed_code']);

    // Store result for post-processing in the finished callback.
    $context['results'][] = l($node->title, 'node/' . $node->nid);

    // Update our progress information.
    $context['sandbox']['progress']++;
  }

  // Inform the batch engine that we are not finished,
  // and provide an estimation of the completion level we reached.
  if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
  }
}

/**
 * Batch 'finished' callback.
 */
function ooyala_import_batch_finished($success, $results, $operations) {
  if (!$success) {
    drupal_set_message(t('An error occurred and processing did not complete.'), 'error');
  }
  $message = format_plural(count($results), '1 item successfully processed:', '@count items successfully processed:');
  $message .= theme('item_list', $results);
  drupal_set_message($message);
}

/**
 * Check to see if this video has already been imported.
 *
 * @params $type_name
 *   The machine name of the content type.
 * @param $keys
 *   An array of video keys to fetch the imported status for.
 *
 * @return
 *   An array of node IDs, keyed by the embed code of the video associated with
 *   the node.
 */
function ooyala_get_status($type_name, $keys = array()) {
  if (empty($keys)) {
    return;
  }

  $return = array();
  $table = 'content_type_'. $type_name;
  $field = ooyala_field_names($type_name);
  $field_name = $field[0] .'_value';

  $placeholders = db_placeholders($keys, "varchar");
  $sql = "SELECT nid, $field_name FROM {" . $table . "} WHERE $field_name IN ($placeholders)";
  $result = db_query($sql, $keys);

  while ($row = db_fetch_array($result)) {
    $return[$row[$field_name]] = $row['nid'];
  }

  return $return;
}

